from __future__ import annotations

import hashlib
import json
import logging
import os
from dataclasses import dataclass

PRODUCTION_URL: str = "https://secure.przelewy24.pl/"
SANDBOX_URL: str = "https://sandbox.przelewy24.pl/"
P24_TEST_CONNECTION: str = "api/v1/testAccess"
P24_TRANSACTION_REQUEST: str = "trnRequest"
P24_TRANSACTION_REGISTER: str = "api/v1/transaction/register"
P24_TRANSACTION_VERIFY: str = "api/v1/transaction/verify"

logger = logging.getLogger(__name__)


@dataclass
class Endpoints:
    testConnection: str
    transactionRequest: str
    transactionRegister: str
    transactionVerify: str


@dataclass(init=False)
class Przelewy24Config:

    pos_id: int
    merchant_id: int
    crc: str
    api_key: str
    endpoints: Endpoints

    def __init__(
        self, pos_id: int, merchant_id: int, crc: str, api_key: str, sandbox: bool
    ):
        self.pos_id = pos_id
        self.merchant_id = merchant_id
        self.crc = crc
        self.api_key = api_key
        base_url = SANDBOX_URL if sandbox else PRODUCTION_URL

        logger.info(
            f"Przelewy24 provider configured: base_url={base_url} pos_id={pos_id} merchant_id={merchant_id} crc={crc} api_key={api_key}"
        )

        self.endpoints = Endpoints(
            testConnection=f"{base_url}{P24_TEST_CONNECTION}",
            transactionRequest=f"{base_url}{P24_TRANSACTION_REQUEST}",
            transactionRegister=f"{base_url}{P24_TRANSACTION_REGISTER}",
            transactionVerify=f"{base_url}{P24_TRANSACTION_VERIFY}",
        )

    def generate_sign(self, **kwargs) -> str:
        return hashlib.sha384(
            json.dumps({**kwargs, **{"crc": self.crc}}).replace(" ", "").encode("utf-8")
        ).hexdigest()

    @classmethod
    def from_env(cls, prefix: str = "PAYMENTS_") -> Przelewy24Config:
        return cls(
            pos_id=int(os.getenv(f"{prefix}P24_POS_ID", 0)),
            merchant_id=int(os.getenv(f"{prefix}P24_MERCHANT_ID", 0)),
            crc=os.getenv(f"{prefix}P24_CRC", "provide P24 CRC"),
            api_key=os.getenv(f"{prefix}P24_API_KEY", "provide P24 API KEY"),
            sandbox=bool(int(os.getenv(f"{prefix}P24_SANDBOX", "1")) != 0),
        )



from decimal import Decimal

from django import forms
from payments.models import BasePayment

from .config import Przelewy24Config


class ProcessForm(forms.Form):
    merchantId = forms.IntegerField()
    posId = forms.IntegerField()
    sessionId = forms.CharField()
    amount = forms.IntegerField()
    originAmount = forms.IntegerField()
    currency = forms.CharField()
    orderId = forms.IntegerField()
    methodId = forms.IntegerField()
    statement = forms.CharField()
    sign = forms.CharField()

    def __init__(self, payment: BasePayment, config: Przelewy24Config, **kwargs):
        super().__init__(**kwargs)
        self.config = config
        self.payment = payment

    def clean(self):
        cleaned_data = super().clean()
        sign = cleaned_data.get("sign", "-1")
        sessionId = cleaned_data.get("sessionId", "-1")
        generated_sign = self.config.generate_sign(
            merchantId=cleaned_data.get("merchantId", 0),
            posId=cleaned_data.get("posId", 0),
            sessionId=cleaned_data.get("sessionId", ""),
            amount=cleaned_data.get("amount", 0),
            originAmount=cleaned_data.get("originAmount", 0),
            currency=cleaned_data.get("currency", ""),
            orderId=cleaned_data.get("orderId", 0),
            methodId=cleaned_data.get("methodId", 0),
            statement=cleaned_data.get("statement", ""),
        )

        if sign != generated_sign:
            raise forms.ValidationError(
                f"Incorrect hash: p24_sign={sign} sign={generated_sign}"
            )

        if sessionId != str(self.payment.pk):
            raise forms.ValidationError(
                f"Incorect payment ID: sessionId: {sessionId}, paymentId: {self.payment.pk}"
            )

        return cleaned_data

    def save(self, *args, **kwargs):
        self.payment.transaction_id = self.cleaned_data["statement"]
        self.payment.captured_amount = (
            self.payment.captured_amount + Decimal(self.cleaned_data["amount"]) / 100
        )
        self.payment.save()



import hashlib
import json
import logging
from dataclasses import asdict, dataclass

import requests

from .config import Przelewy24Config

logger = logging.getLogger(__name__)


@dataclass
class Transaction:
    amount: int
    sessionId: str
    currency: str
    description: str
    email: str
    country: str
    language: str


@dataclass
class TransactionDTO:
    merchantId: int
    posId: int
    sessionId: str
    amount: int
    currency: str
    description: str
    email: str
    country: str
    language: str
    urlReturn: str
    urlStatus: str
    sign: str
    # cart: List[ItemDTO]

    @classmethod
    def create_from(
        cls,
        transaction: Transaction,
        config: Przelewy24Config,
        sign: str,
        success_url: str,
        status_url: str,
    ):
        return cls(
            merchantId=config.merchant_id,
            posId=config.merchant_id,
            sessionId=transaction.sessionId,
            amount=transaction.amount,
            currency=transaction.currency,
            description=transaction.description,
            email=transaction.email,
            country=transaction.country,
            language=transaction.language,
            urlReturn=success_url,
            urlStatus=status_url,
            sign=sign,
        )


@dataclass
class VerifyDTO:
    merchantId: int
    posId: int
    sessionId: str
    amount: int
    currency: str
    orderId: int
    sign: str

    @classmethod
    def create_from(
        cls,
        *,
        orderId: int,
        transaction: Transaction,
        config: Przelewy24Config,
        sign: str,
    ):
        return cls(
            merchantId=config.merchant_id,
            posId=config.merchant_id,
            sessionId=transaction.sessionId,
            amount=transaction.amount,
            currency=transaction.currency,
            orderId=orderId,  # TODO
            sign=sign,
        )


class Przelewy24API:
    def __init__(self, config: Przelewy24Config, session=None):
        self._http = session or requests.session()
        self._config = config
        super().__init__()

    def _do(self, method: str, endpoint: str, data=None):
        response = self._http.request(
            method=method,
            url=endpoint,
            json=data,
            auth=(str(self._config.pos_id), str(self._config.api_key)),
        )
        logger.debug(
            "%s %s: status_code=%s content=%s",
            method,
            endpoint,
            response.status_code,
            response.content.decode("utf-8"),
        )
        if response.status_code != 200:
            raise RuntimeError(
                f"Przelewy24 returns {response.status_code} instead of 200: {response.content}"
            )
        return response.json()

    def _create_sha386_sign(self, **kwargs) -> str:
        return hashlib.sha384(
            json.dumps(kwargs).replace(" ", "").encode("utf-8")
        ).hexdigest()

    def testConnection(self) -> bool:
        response = self._do("GET", self._config.endpoints.testConnection)
        return response["data"]

    def register(
        self, *, transaction: Transaction, success_url: str, status_url: str
    ) -> str:
        sign = self._config.generate_sign(
            sessionId=transaction.sessionId,
            merchantId=self._config.merchant_id,
            amount=transaction.amount,
            currency=transaction.currency,
        )

        transaction = TransactionDTO.create_from(
            transaction, self._config, sign, success_url, status_url
        )
        payload = asdict(transaction)
        response = self._do("POST", self._config.endpoints.transactionRegister, payload)
        token = response["data"]["token"]
        return f"{self._config.endpoints.transactionRequest}/{token}"

    def verify(self, *, transaction: Transaction, orderId: int) -> bool:
        sign = self._config.generate_sign(
            sessionId=transaction.sessionId,
            orderId=orderId,
            amount=transaction.amount,
            currency=transaction.currency,
        )
        verify = VerifyDTO.create_from(
            orderId=orderId, transaction=transaction, config=self._config, sign=sign
        )
        payload = asdict(verify)
        response = self._do("PUT", self._config.endpoints.transactionVerify, payload)
        return response["data"]["status"] == "success"



import json
import logging
from decimal import Decimal

import requests
from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponse, HttpResponseForbidden
from django.http.response import HttpResponseBadRequest
from payments import PaymentStatus
from payments.core import BasicProvider
from payments.forms import PaymentForm
from payments.models import BasePayment

from payments_przelewy24.api import Transaction
from payments_przelewy24.forms import ProcessForm

from .api import Przelewy24API, Przelewy24Config

CENTS = Decimal("0.01")

logger = logging.getLogger(__name__)


def _create_transaction_from_payment(payment: BasePayment):
    return Transaction(
        sessionId=str(payment.pk),
        amount=int(payment.total / CENTS),
        currency=payment.currency,
        description=payment.description,
        email=payment.billing_email,
        country=payment.billing_country_code,
        language="pl",  # TODO,
    )


class Przelewy24Provider(BasicProvider):
    """Payment provider for Przelewy24.pl
    This backend implements payments using a popular Polish gateway, `Przelewy24.pl
    <http://www.Przelewy24.pl>`_.
    """

    _method = "post"

    def __init__(self, config: Przelewy24Config, **kwargs):
        self._api = Przelewy24API(config)
        self._config = config
        super().__init__(**kwargs)
        if not self._capture:
            raise ImproperlyConfigured("Przelewy24 does not support pre-authorization.")

    def get_action(self, payment):
        url = self._api.register(
            transaction=_create_transaction_from_payment(payment),
            success_url=payment.get_success_url(),
            status_url=self.get_return_url(payment),
        )
        logger.debug(f"Transaction registered: url={url}")
        return url

    def get_form(self, payment, data=None):
        form = super().get_form(payment, data)
        return form

    def get_hidden_fields(self, payment):
        return {}

    def get_payment_response(self, payment, extra_data=None):
        post = self.get_product_data(payment, extra_data)
        return requests.post(self.endpoint, data=post)

    def process_data(self, payment, request):
        logging.info("Process Przelewy24's notification: body={request.body}")
        try:
            data = json.loads(request.body.decode("utf-8"))
            form = ProcessForm(payment=payment, config=self._config, data=data)
            if form.is_valid():
                orderId = data["orderId"]
                self._api.verify(
                    transaction=_create_transaction_from_payment(payment),
                    orderId=orderId,
                )
                form.save()
                payment.change_status(PaymentStatus.CONFIRMED)
            else:
                error_str = ", ".join([f"{k}: {v}" for k, v in form.errors.items()])
                logger.error(error_str)
                return HttpResponseBadRequest("Failed - incorrect data")
        except Exception as e:
            logger.error(f"{str(e)}, {request.body.decode('utf-8')}")
            return HttpResponseBadRequest("Failed")
        return HttpResponse("OK")
